"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: shpTools.py
@Date: Create in 2021/1/22 13:43
@Description: 对geojson文件的操作类
@ Software: PyCharm

===================================
"""

import json
import os

import geopandas as gpd


class GeoJsonHelper:

    def __init__(self):
        pass

    @staticmethod
    # 合并 geojson
    def merge_geojsons(geojson_files_path, merge_result_file):
        index = 0
        all_data = {}
        for file_name in os.listdir(geojson_files_path):
            geojson_file = os.path.join(geojson_files_path, file_name)
            with open(geojson_file, 'r', encoding='utf-8') as fp:
                each_data = json.load(fp)
            if index == 0:
                all_data = each_data
            else:
                all_data["features"] += each_data["features"]
        result_file = open(merge_result_file, "w")
        json.dump(all_data, result_file)

    @staticmethod
    def convert_geojson_to_shp(geojson_path, shp_path):
        # 读取 GeoJSON 文件
        gdf = gpd.read_file(geojson_path, engine='pyogrio')
        # 保存为 Shapefile
        gdf.to_file(shp_path, driver='ESRI Shapefile', engine='pyogrio')


if __name__ == '__main__':
    geojson_path = "e:\\tttt.geojson"
    shp_path = "e:\\tttt.shp"
    GeoJsonHelper.convert_geojson_to_shp(geojson_path, shp_path)



"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: coordTools.py
@Date: Create in 2021/1/28 10:04
@Description: 坐标转换
（wgs84ll即GPS经纬度），2（gcj02ll即国测局经纬度坐标），3（bd09ll即百度经纬度坐标），4（bd09mc即百度米制坐标）
@ Software: PyCharm
===================================
"""

import math


class coordConverter:
    def __init__(self):
        self.x_pi = float(3.14159265358979324 * 3000.0 / 180.0)
        # //pai
        self.pi = float(3.1415926535897932384626)
        # //离心率
        self.ee = float(0.00669342162296594323)
        # //长半轴
        self.a = float(6378245.0)

    # //百度转国测局
    def bd09togcj02(self, bd_lon, bd_lat):
        x = (bd_lon - 0.0065)
        y = (bd_lat - 0.006)
        z = (math.sqrt(x * x + y * y)) - (0.00002 * math.sin(y * self.x_pi))
        theta = math.atan2(y, x) - 0.000003 * math.cos(x * self.x_pi)
        gg_lng = z * math.cos(theta)
        gg_lat = z * math.sin(theta)

        return gg_lng, gg_lat

    # //国测局转百度
    def gcj02tobd09(self, lng, lat):
        z = math.sqrt(lng * lng + lat * lat) + 0.00002 * math.sin(lat * self.x_pi)
        theta = math.atan2(lat, lng) + 0.000003 * math.cos(lng * self.x_pi)
        bd_lng = z * math.cos(theta) + 0.0065
        bd_lat = z * math.sin(theta) + 0.006
        return bd_lng, bd_lat

    # //国测局转84
    def gcj02towgs84(self, lng, lat):
        dlat = self.transformlat(lng - 105.0, lat - 35.0)
        dlng = self.transformlng(lng - 105.0, lat - 35.0)
        radlat = lat / 180.0 * self.pi
        magic = math.sin(radlat)
        magic = 1 - self.ee * magic * magic
        sqrtmagic = math.sqrt(magic)
        dlat = (dlat * 180.0) / ((self.a * (1 - self.ee)) / (magic * sqrtmagic) * self.pi)
        dlng = (dlng * 180.0) / (self.a / sqrtmagic * math.cos(radlat) * self.pi)
        mglat = lat + dlat
        mglng = lng + dlng
        return [lng * 2 - mglng, lat * 2 - mglat]

    # //经度转换
    def transformlat(self, lng, lat):
        ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + 0.1 * lng * lat + 0.2 * math.sqrt(abs(lng))
        ret += (20.0 * math.sin(6.0 * lng * self.pi) + 20.0 * math.sin(2.0 * lng * self.pi)) * 2.0 / 3.0
        ret += (20.0 * math.sin(lat * self.pi) + 40.0 * math.sin(lat / 3.0 * self.pi)) * 2.0 / 3.0
        ret += (160.0 * math.sin(lat / 12.0 * self.pi) + 320 * math.sin(lat * self.pi / 30.0)) * 2.0 / 3.0
        return ret

    # //纬度转换
    def transformlng(self, lng, lat):
        ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + 0.1 * lng * lat + 0.1 * math.sqrt(abs(lng))
        ret += (20.0 * math.sin(6.0 * lng * self.pi) + 20.0 * math.sin(2.0 * lng * self.pi)) * 2.0 / 3.0
        ret += (20.0 * math.sin(lng * self.pi) + 40.0 * math.sin(lng / 3.0 * self.pi)) * 2.0 / 3.0
        ret += (150.0 * math.sin(lng / 12.0 * self.pi) + 300.0 * math.sin(lng / 30.0 * self.pi)) * 2.0 / 3.0
        return ret

    def getWgs84xy(self, x, y):
        # //先转 国测局坐标
        doubles_gcj = self.bd09togcj02(x, y)
        # //（x 117.   y 36. ）
        # //国测局坐标转wgs84
        doubles_wgs84 = self.gcj02towgs84(doubles_gcj[0], doubles_gcj[1])
        # //返回 纠偏后 坐标
        return doubles_wgs84



#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2022/10/28 18:56
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : gisTools.py
# @Descr   : 
# @Software: PyCharm

# !/usr/bin/python3.9
# -*- coding: utf-8 -*-
"""
    地理中经常使用的数学计算，把地球简化成了一个标准球形，若是想要推广到任意星球能够改为类的写法，而后修改半径便可
"""
import math
import re

import numpy as np
from osgeo import gdal
from osgeo import osr
from pyproj import Transformer
from shapely.geometry import Polygon


class GISHelper:
    earth_radius = 6370.856  # 地球平均半径，单位km，最简单的模型每每把地球当作完美的球形，这个值就是常说的RE
    math_2pi = math.pi * 2
    pis_per_degree = math_2pi / 360  # 角度一度所对应的弧度数，360对应2*pi

    def __init__(self):
        pass

    # 纬度差对应的KM
    def lat_degree2km(self, dif_degree, radius=earth_radius):
        """
        经过圆环求法，纯纬度上，度数差转距离(km)，与中间点所处的地球上的位置关系不大
        :param dif_degree: 度数差, 经验值0.0001对应11.1米的距离
        :param radius: 圆环求法的等效半径，纬度距离的等效圆环是经线环，因此默认都是earth_radius
        :return: 这个度数差dif_degree对应的距离，单位km
        """
        return radius * dif_degree * self.pis_per_degree

    # KM对应的纬度差
    def lat_km2degree(self, dis_km, radius=earth_radius):
        """
        经过圆环求法，纯纬度上，距离值转度数(diff)，与中间点所处的地球上的位置关系不大
        :param dis_km: 输入的距离，单位km，经验值111km相差约(接近)1度
        :param radius: 圆环求法的等效半径，纬度距离的等效圆环是经线环，因此默认都是earth_radius
        :return: 这个距离dis_km对应在纯纬度上差多少度
        """
        return dis_km / radius / self.pis_per_degree

    # 经度差对应的KM，需要提供中央纬度
    def lng_degree2km(self, dif_degree, center_lat):
        """
        经过圆环求法，纯经度上，度数差转距离(km)，纬度的高低会影响距离对应的经度角度差，具体表达式为：
        :param dif_degree: 度数差
        :param center_lat: 中心点的纬度，默认22为深圳附近的纬度值；为0时表示赤道，赤道的纬线环半径使得经度计算和上面的纬度计算基本一致
        :return: 这个度数差dif_degree对应的距离，单位km
        """
        # 修正后，中心点所在纬度的地表圆环半径
        real_radius = self.earth_radius * math.cos(center_lat * self.pis_per_degree)
        return self.lat_degree2km(dif_degree, real_radius)

    # KM对应的经度差，需要提供中央纬度
    def lng_km2degree(self, dis_km, center_lat):
        """
        纯经度上，距离值转角度差(diff)，单位度数。
        :param dis_km: 输入的距离，单位km
        :param center_lat: 中心点的纬度，默认22为深圳附近的纬度值；为0时表示赤道。
             赤道、中国深圳、中国北京、对应的修正系数分别约为： 1  0.927  0.766
        :return: 这个距离dis_km对应在纯经度上差多少度
        """
        # 修正后，中心点所在纬度的地表圆环半径
        real_radius = self.earth_radius * math.cos(center_lat * self.pis_per_degree)
        return self.lat_km2degree(dis_km, real_radius)

    # 两点（经纬度）之间的KM
    def ab_distance(self, a_lat, a_lng, b_lat, b_lng):
        """
        计算经纬度表示的ab两点的距离，这是种近似计算，当两点纬度差距不大时更为准确，产生近似的缘由也是来主要自于center_lat
        :param a_lat: a点纬度
        :param a_lng: a点经度
        :param b_lat: b点纬度
        :param b_lng: b点纬度
        :return:
        """
        center_lat = .5 * a_lat + .5 * b_lat
        lat_dis = self.lat_degree2km(abs(a_lat - b_lat))
        lng_dis = self.lng_degree2km(abs(a_lng - b_lng), center_lat)
        return math.sqrt(lat_dis ** 2 + lng_dis ** 2)

    # 构建矩形
    def get_rect_by_coords(self, coords):
        square = Polygon(
            ((coords[0], coords[2]), (coords[0], coords[3]), (coords[1], coords[3]), (coords[1], coords[2])))
        return square

    # 判断两个多边形是否相交
    # shaplely方法
    def is_overlap_of_polygons(self, poly1, poly2):
        return poly1.intersects(poly2) and not poly1.crosses(poly2) and not poly1.contains(poly2)

    # 通过经纬度得到像素坐标,根据地图高度宽度及地图经纬度范围
    def get_pixel_by_lnglat(self, point_x: float, point_y: float, map_boundary: str, map_witdh: int,
                            map_height: int) -> str:
        minx = float(map_boundary.split(",")[0])
        miny = float(map_boundary.split(",")[1])
        maxx = float(map_boundary.split(",")[2])
        maxy = float(map_boundary.split(",")[3])
        # 通过地图长宽和经纬度坐标获取像素坐标
        pixel_x = map_witdh / (maxx - minx) * (point_x - minx)
        pixel_y = map_height / (maxy - miny) * (maxy - point_y)
        return round(pixel_x), round(pixel_y)

    # 通过中心点坐标(经纬度)，图片宽度，图片高度，分辨率得到图片四个点坐标
    def get_box_by_center(self, data_centerpt: str, data_resolution: float, data_image_width: int,
                          data_image_height: int) -> str:

        data_minx = float(data_centerpt.split(",")[0]) - self.lng_km2degree(
            float(data_image_width) / 2 * float(data_resolution) / 1000, float(data_centerpt.split(",")[1]))
        data_maxx = float(data_centerpt.split(",")[0]) + self.lng_km2degree(
            float(data_image_width) / 2 * float(data_resolution) / 1000, float(data_centerpt.split(",")[1]))
        data_miny = float(data_centerpt.split(",")[1]) - self.lat_km2degree(
            float(data_image_height) / 2 * float(data_resolution) / 1000, 6370.856)
        data_maxy = float(data_centerpt.split(",")[1]) + self.lat_km2degree(
            float(data_image_height) / 2 * float(data_resolution) / 1000, 6370.856)
        return data_minx, data_maxx, data_miny, data_maxy

    # 4326坐标转3857坐标
    # point [[123.5,23.6]]
    @staticmethod
    def convert_point_from_wgs84_to_mercator(point):
        return_point = None
        # 地球半径
        earthRad = 6378137.0
        lng = point[0] * math.pi / 180 * earthRad
        param = point[1] * math.pi / 180
        lat = earthRad / 2 * math.log((1.0 + math.sin(param)) / (1.0 - math.sin(param)))
        return_point = [lng, lat]
        return return_point

    # 4326坐标转3857坐标
    # point [[350388,3957319.5]]
    @staticmethod
    def convert_point_from_mercator_to_wgs84(point):
        return_point = None
        lng = point[0] / 20037508.34 * 180
        lat = point[1] / 20037508.34 * 180
        lat = 180 / math.pi * (2 * math.atan(math.exp(lat * math.pi / 180)) - math.pi / 2)
        return_point = [lng, lat]
        return return_point

    # 投影坐标(米)到地理坐标（度）
    # proj_wkt： gdalinfo -json **.tif 得到json["coordinateSystem"]["wkt"]
    def convert_point_from_proj_to_geo(self, proj_wkt, x, y):
        # 地理坐标系是球面,投影坐标系是平面;
        # 地理坐标系的单位为“度”,投影坐标系的单位为“米”;
        prosrs = osr.SpatialReference()
        prosrs.ImportFromWkt(proj_wkt)
        geosrs = prosrs.CloneGeogCS()
        ct = osr.CoordinateTransformation(prosrs, geosrs)
        coords = ct.TransformPoint(x, y)
        # # 逆变换,实现投影到地理
        # p = Proj(projection)
        # lon, lat = p(x, y, inverse=True)
        return coords[:2][0], coords[:2][1]

    # 地理坐标（度）到投影坐标（米）
    def convert_point_from_geo_to_proj(self, proj_wkt, lon, lat):
        prosrs = osr.SpatialReference()
        prosrs.ImportFromWkt(proj_wkt)
        geosrs = prosrs.CloneGeogCS()
        ct = osr.CoordinateTransformation(geosrs, prosrs)
        coords = ct.TransformPoint(lon, lat)
        return coords[:2][0], coords[:2][1]

    # 投影坐标（米）到投影坐标（米）
    def convert_point_from_proj_to_proj(self, proj_wkt_source, proj_wkt_target, x, y):
        pass
        prosrs_source = osr.SpatialReference()
        prosrs_source.ImportFromWkt(proj_wkt_source)
        prosrs_target = osr.SpatialReference()
        prosrs_target.ImportFromWkt(proj_wkt_target)
        ct = osr.CoordinateTransformation(prosrs_source, prosrs_target)
        coords = ct.TransformPoint(x, y)
        return coords[:2][0], coords[:2][1]

    # 坐标转换（从一个坐标系到另一个坐标系），多个点
    # points:[[x1,y1],[x2,y2]....]
    # convert_method:proj_2_geo\geo_2_proj\proj_2_proj
    def convert_points_by_projwkt(self, points, proj_wkt, proj2_wkt, convert_method):
        try:
            point_list = []
            # 循环坐标点数组
            for point in points:
                x = point[0]
                y = point[1]
                if convert_method == "proj_2_geo":
                    x_convert, y_convert = self.convert_point_from_proj_to_geo(proj_wkt, x, y)
                if convert_method == "geo_2_proj":
                    x_convert, y_convert = self.convert_point_from_geo_to_proj(proj_wkt, x, y)
                if convert_method == "proj_2_proj":
                    x_convert, y_convert = self.convert_point_from_proj_to_proj(proj_wkt, proj2_wkt, x, y)

                point_list.append([x_convert, y_convert])

        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 输出异常信息，错误行号等
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

        return point_list

    # 针对WGS_1984_UTM_Zone_57N这种WGS84 UTM的分带投影做坐标转换
    # 从投影坐标米到地理坐标---单个点
    # point:[[350388,3957319.5]]
    # proj_wkt:"PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH],AUTHORITY[\"EPSG\",\"32654\"]]"
    def convert_point_from_utmproj_to_geo(self, point, proj_wkt, center_lon, center_lat):
        try:
            return_point = None
            point_x = point[0][0]
            point_y = point[0][1]
            proj_wkt = proj_wkt.upper()

            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            # 如果是经纬度，就不用转
            if proj_wkt.strip().startswith("GEOGCS"):
                return_point = point
            else:
                proj = re.findall("PROJCS(.*?)GEOGCS", proj_wkt)[0].lstrip("[")[:-1]
                epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
                if epsg != -1:
                    print("转换前的espg:{}".format(epsg))
                    transformer = Transformer.from_crs("epsg:" + str(epsg), "epsg:4326")
                    lat, lon = transformer.transform(point_x, point_y)
                    return_point = [[lon, lat]]
                else:
                    print("坐标转换失败，通过投影没有获取到EPSG，请检查数据")
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 打印输出异常信息
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

        return return_point

    # 转换坐标---从投影坐标米到地理坐标--多个点
    # points:[[350388,3957319.5],[352344,3966458.5]]
    # proj_wkt:"PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH],AUTHORITY[\"EPSG\",\"32654\"]]"
    def convert_points_from_utmproj_to_geo(self, points, proj_wkt, center_lon, center_lat):
        return_point_list = None
        try:
            res = ""
            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            # 参数3: WGS84/UTM LJY,UTM无分带，强制对应为3857
            proj_wkt = proj_wkt.upper()
            print("投影信息：{}".format(proj_wkt))
            # 如果是经纬度，就不用转
            if proj_wkt.strip().startswith("GEOGCS"):
                return_point_list = points
            else:
                proj = re.findall("PROJCS(.*?)GEOGCS", proj_wkt)[0].lstrip("[")[:-1]
                epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
                print("转换前的espg:{}".format(epsg))
                transformer = Transformer.from_crs("epsg:" + str(epsg), "epsg:4326")
                point_list = []
                for point in points:
                    x = point[0]
                    y = point[1]
                    lat, lon = transformer.transform(x, y)
                    point_list.append([lon, lat])
                return_point_list = point_list
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 输出异常信息，错误行号等
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数
        return return_point_list

    # 针对WGS_1984_UTM_Zone_57N这种WGS84 UTM的分带投影做坐标转换
    # 地理坐标（经纬度）到投影坐标（米）--单个点
    def convert_point_from_geo_to_utmproj(self, point, proj_wkt, center_lon, center_lat):
        return_point = None
        try:
            lon = point[0][0]
            lat = point[0][1]
            proj_wkt = proj_wkt.upper()
            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            # 投影1：PROJCS[\"WGS_1984_UTM_Zone_54N\",GEOGCS
            # 投影2：PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS
            proj = re.findall("PROJCS(.*?)GEOGCS", proj_wkt)[0].lstrip("[")[:-1]
            epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
            if epsg != -1:
                print("转换前的espg:{}".format(epsg))
                # 建立转换器
                transformer = Transformer.from_crs("epsg:4326", "epsg:" + str(epsg))
                # lat, lon = transformer.transform(point_x, point_y)
                point_x, point_y = transformer.transform(lat, lon)
                return_point = [[point_x, point_y]]
            else:
                print("坐标转换失败，通过投影没有获取到EPSG，请检查数据")
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 打印输出异常信息
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

        return return_point

    # 转换坐标---地理坐标（经纬度）到投影坐标（米）---多个点
    # point:[[350388,3957319.5],[352344,3966458.5]]
    # proj_txt:"PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",141],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH],AUTHORITY[\"EPSG\",\"32654\"]]"
    def convert_points_from_geo_to_utmproj(self, points, proj_wkt, center_lon, center_lat):
        return_point_list = None
        try:
            res = ""
            # 参数1：坐标系WKID WGS_1984_UTM_Zone_54N 对应 32654
            # 参数2：WGS84地理坐标系统 对应 4326
            proj_wkt = proj_wkt.upper()
            # 通过正则表达式得到坐标系
            # 投影1：PROJCS[\"WGS_1984_UTM_Zone_54N\",GEOGCS
            # 投影2：PROJCS[\"WGS 84 / UTM zone 54N\",GEOGCS
            proj = re.findall("PROJCS(.*?)GEOGCS", proj_wkt)[0].lstrip("[")[:-1]
            epsg = self.get_utm_epsg_by_projection_info(proj, center_lon, center_lat)
            print("转换前的espg:{}".format(epsg))
            transformer = Transformer.from_crs("epsg:4326", "epsg:" + str(epsg))
            point_list = []
            # 循环坐标点数组
            for point in points:
                lon = point[0]
                lat = point[1]
                x, y = transformer.transform(lat, lon)
                point_list.append([x, y])
            return_point_list = point_list
        except Exception as exp:
            print("坐标转换失败：" + str(exp))
            # 输出异常信息，错误行号等
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数
        return return_point_list

    # 通过投影信息和中央经纬度得到EPSG
    def get_utm_epsg_by_projection_info(self, projection, center_lon, center_lat):
        epsg_code = -1
        try:
            print("转换前的proj:{}".format(projection))
            utm_code = projection.replace(" ", "").replace("_", "").lower()
            # wgs84 utm分带投影的epsg计算
            if "utmzone" in utm_code and ("wgs84" in utm_code or "wgs1984" in utm_code):
                if len(re.findall("zone(.*?)n", utm_code)) > 0:
                    # 北半球 epsg=32600+wgs84 utm分度带号
                    utm_code_zone = int(re.findall("zone(.*?)n", utm_code)[0])
                    epsg_code = 32600 + utm_code_zone
                if len(re.findall("zone(.*?)s", utm_code)) > 0:
                    # 南半球 epsg=32700+wgs84 utm分度带号
                    utm_code_zone = int(re.findall("zone(.*?)s", utm_code)[0])
                    epsg_code = 32700 + utm_code_zone
            # utm没有分带投影的epsg计算
            elif "utm" in utm_code and "utmzone" not in utm_code:
                # 东半球
                if float(center_lon) > 0:
                    utm_code_zone = 31 + int(float(center_lon) / 6)
                # 西半球
                if float(center_lon) < 0:
                    utm_code_zone = 30 - int(abs(float(center_lon) / 6))
                # 北半球
                if float(center_lat) > 0:
                    epsg_code = 32600 + utm_code_zone
                # 南半球
                if float(center_lat) < 0:
                    epsg_code = 32700 + utm_code_zone

        except Exception as exp:
            print("通过投影获取EPSG失败：" + str(exp))
            # epsg_code=3857
            print(exp)
            print(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            print(exp.__traceback__.tb_lineno)  # 发生异常所在的行数
        return int(epsg_code)

    # 计算多边形坐标的四至范围
    # 多边形坐标为[[point1x,point1y],[point2x,point2y],...[pointnx,pointny],[point1x,point1y]]
    @staticmethod
    def get_box_of_polygon_points(points):
        points_np = np.array(points)

        cols_min = np.min(points_np, axis=0)
        cols_max = np.max(points_np, axis=0)
        minx = cols_min[:1].tolist()[0]
        miny = cols_min[1:2].tolist()[0]
        maxx = cols_max[:1].tolist()[0]
        maxy = cols_max[1:2].tolist()[0]
        # minx = np.min(points_np[:, 0])
        # maxx = np.max(points_np[:, 0])
        # miny = np.min(points_np[:, 1])
        # maxy = np.max(points_np[:, 1])
        return minx, miny, maxx, maxy

    # 得到tif文件的投影信息
    def get_source_projection(self, tif_file):
        dataset = gdal.Open(tif_file)
        return dataset.GetProjection()


if __name__ == '__main__':
    gisHelper = GISHelper()
    # print(gisHelper.lng_km2degree(0.5, 45))
    # dataset = gdal.Open("E:\\data\\123.tif")
    # print(dataset.GetProjection())
    # print(dataset.GetGeoTransform())

    print("---------------------------------")

    # dataset = gdal.Open("E:\\data\\sar1234\\1234.tiff")
    # print(dataset.GetProjection())
    # print(dataset.GetGeoTransform())

    # 得到原始tif的投影信息
    # source_projection = gisHelper.get_source_projection("E:\\data\\123.tif")
    # source_projection = gisHelper.get_source_projection("E:\\desktop\\DEM\\ribenxibu.tif")
    # # 输入原始tif的投影信息，需要转换的点的投影坐标（米）
    # result = gisHelper.convert_points_from_meter_to_lnglat(source_projection, 350388, 3957319.5)
    # # 输出 转换后的经纬度值（度）
    # print("lon:" + str(result[0]))
    # print("lat:" + str(result[1]))
    point = [113, 23]
    print(gisHelper.convert_point_from_wgs84_to_mercator(point))
    point = [12604705.942522367, 2632018.637586424]
    print(gisHelper.convert_point_from_mercator_to_wgs84(point))



"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: geocodeTools.py
@Date: Create in 2021/1/28 9:50
@Description: 地址匹配(百度，高德，四维等）
http://lbsyun.baidu.com/index.php?title=webapi/guide/webservice-geocoding
（wgs84ll即GPS经纬度），2（gcj02ll即国测局经纬度坐标），3（bd09ll即百度经纬度坐标），4（bd09mc即百度米制坐标）
@ Software: PyCharm
===================================
"""
import json
from time import sleep

import requests


class AddressMatcher:
    # 初始化
    def __init__(self, baidu_key_list, gaode_key_list, siwei_key_list):
        self.baidu_key_list = baidu_key_list
        self.gaode_key_list = gaode_key_list
        self.siwei_key_list = siwei_key_list
        self.baidu_key_index = 0
        self.baidu_key = self.baidu_key_list[self.baidu_key_index]
        self.baidu_key_status = "配额正常"

    # 正向匹配，从地址名/POI名到坐标
    # TODO:高德，四维的接口加入
    def geocoder_forward(self, query_address):
        # 先用百度做匹配
        result_obj = self.get_geocoder_forward_result_by_baidu(query_address)
        # 配额用完，切换下一个账号许可，再执行一次
        if self.baidu_key_status == "配额用完":
            result_obj = self.get_geocoder_forward_result_by_baidu(query_address)
        # 针对百度没有匹配上的再用高德匹配
        # if result_obj.get("经度") is None:
        #     result_obj = self.get_geocoder_forward_result_by_gaode(query_address)
        # # 针对百度和高德都没有匹配上的再用四维匹配
        # if result_obj.get("经度") is None:
        #     result_obj = self.get_geocoder_forward_result_by_siwei(query_address)
        return result_obj

    # 通过百度进行正向地址匹配
    def get_geocoder_forward_result_by_baidu(self, query_address):
        sleep(1)
        headers = {
            "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.81 Safari/537.36",
        }
        url = 'http://api.map.baidu.com/geocoding/v3/?address={}&output=json&ak={}&ret_coordtype=wgs84ll'.format(
            query_address, self.baidu_key)
        res = requests.get(url, headers=headers)
        res.encoding = 'utf-8-sig'
        data = json.loads(res.text)
        result_obj = {}
        if data.get("status") == 0:
            self.baidu_key_status = "配额正常"
            result_obj['经度'] = data['result'].get('location').get('lng')
            result_obj['纬度'] = data['result'].get('location').get('lat')
        # 没有查询结果
        elif data.get("status") == 1:
            self.baidu_key_status = "配额正常"
            result_obj['经度'] = None
            result_obj['纬度'] = None
        # 超出配额限制
        elif data.get("status") == 301 or data.get("status") == 302:
            result_obj['经度'] = None
            result_obj['纬度'] = None
            print("------------------------------{}配额用完".format(self.baidu_key))
            self.baidu_key_index += 1
            self.baidu_key = self.baidu_key_list[self.baidu_key_index]
            self.baidu_key_status = "配额用完"
            print("-------------------------------切换到新许可{}".format(self.baidu_key))
        return result_obj

    # 通过高德进行正向地址匹配
    def get_geocoder_forward_result_by_gaode(self):
        result_obj = {}
        # result_obj['经度'] = data['result'].get('location').get('lng')
        # result_obj['纬度'] = data['result'].get('location').get('lat')
        return result_obj

    # 通过四维进行正向地址匹配
    def get_geocoder_forward_result_by_siwei(self):
        result_obj = {}
        # result_obj['经度'] = data['result'].get('location').get('lng')
        # result_obj['纬度'] = data['result'].get('location').get('lat')
        return result_obj

    # 逆向匹配，从坐标到地址
    def geocoder_reverse(self):
        pass

    # 通过百度逆向地址匹配
    def get_geocoder_reverse_result_by_baidu(self, query_lat, query_lon):
        result_obj = {}
        url = "http://api.map.baidu.com/reverse_geocoding/v3/?ak={}&output=json&coordtype=wgs84ll&location=".format(
            self.baidu_key) + str(
            query_lat) + "," + str(query_lon)
        payload = {}
        headers = {
            "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.81 Safari/537.36",
        }
        response = requests.request("GET", url, headers=headers, data=payload)
        obj_json = json.loads(response.text)
        result_obj["省"] = obj_json.get("result").get("addressComponent").get("province")
        result_obj["地市"] = obj_json.get("result").get("addressComponent").get("city")
        result_obj["区县"] = obj_json.get("result").get("addressComponent").get("district")
        return result_obj



"""
===================================
#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: arcgisTools.py
@Date: Create in 2021/2/5 9:37
@Description:
arcgis的Python
arcgis desktop只支持python2.7
需要安装arcgis desktop
配置pycharm的python环境，C:\Python27\ArcGIS10.5/python.exe
# D:\Program Files (x86)\ArcGIS\Desktop10.5\arcpy；
# D:\Program Files (x86)\ArcGIS\Desktop10.5\ArcToolbox\Scripts；
# D:\Program Files (x86)\ArcGIS\Desktop10.5\bin
@ Software: PyCharm
===================================
"""
# import arcpy
#
#
# # ArcGIS Pro的Arcpy原生支持Python3
#
# mypath=r"C:\Users\Administrator\Desktop\huodianchang\shanxi.mxd"
# mxd = arcpy.mapping.MapDocument(mypath)
#
# # df = arcpy.mapping.ListDataFrames(mxd, "Layers")[0]
# # theShape = r"E:\zgl.shp"
# # addLayer = arcpy.mapping.Layer(theShape)
# # arcpy.mapping.AddLayer(df, addLayer, "AUTO_ARRANGE")
# # arcpy.RefreshActiveView()
# # arcpy.RefreshTOC()
# # mxd.save()
#
#
# #图层
# lyr = arcpy.mapping.ListLayers(mxd)[0]
# #数据框
# df = arcpy.mapping.ListDataFrames(mxd)[0]
# rows = arcpy.SearchCursor(lyr)
# for row in rows:
#     geo = row.shape
#     #要素名称字段
#     name = row.getValue("name")
#     #数据框范围以当前要素为中心
#     df.panToExtent(geo.extent)
#     arcpy.mapping.ExportToPNG(mxd, r"C:/Users/Administrator/Desktop/huodianchang/shanxitu/" + name + ".png", 300)





#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2022/10/28 18:56
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : gisTools.py
# @Descr   : 
# @Software: PyCharm


class ProjHelper:

    def __init__(self):
        pass

    @staticmethod
    def convertJgwToGadlTransform(jgwFilePath: str) -> str:
        """
        将世界投影文件jpw格式转为gdal的投影六参数数组
        jgw格式为：0.0000197051\n 0.0000000000\n 0.0000000000\n -0.0000197051\n 104.595572656171356\n 38.815550616869423\n
            #  A 表示影像数据与矢量数据在 X 方向的比例关系;X方向 每像素的精度值
            #  D 表示影像数据与矢量数据在 Y方向的比例关系, 但它为负值, 这是由于空间坐标系与影像数据的存储坐标系在 Y方向上相反, 要匹配, 必须将 E 设为负值;Y方向 每像素的精度值
            #  E、F 表示影像数据的左上角点的像元对应的空间坐标的 X、Y 坐标;
            #  B、C 表示影像数据的旋转参数, 但是在 MapObjects2. 0 中不支持影像数据的旋转, 因此 这两个参数的数值是被忽略的, 缺省记录为 0。
        gdal transform格式为[114.0, 0.000001, 0, 34.0, 0, -0.000001]
            # 左上角点坐标X
            # x方向的分辨
            # 旋转系数，如果为0，就是标准的正北向图像
            # 左上角点坐标Y
            # 旋转系数，如果为0，就是标准的正北向图像
            # Y方向的分辨率

        :param jgwFilePath: 世界投影文件路径
        :return: gdal的几何投影六参数数组
        """
        f = open(jgwFilePath)
        line = f.readline()
        row = 0
        while line:
            row += 1
            if row == 1:
                x_scale_value = line.rstrip("\n")
            if row == 2:
                x_rotate_value = line.rstrip("\n")
            if row == 3:
                y_rotate_value = line.rstrip("\n")
            if row == 4:
                y_scale_value = line.rstrip("\n")
            if row == 5:
                x_leftupper_value = line.rstrip("\n")
            if row == 6:
                y_leftupper_value = line.rstrip("\n")
            line = f.readline()
        f.close()
        gdal_transform = [float(x_leftupper_value), float(x_scale_value), float(x_rotate_value),
                          float(y_leftupper_value), float(y_rotate_value),
                          float(y_scale_value)]
        return gdal_transform



"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: geoserverTools.py
@Date: Create in 2021/1/28 13:14
@Description: GeoServer操作类
@ Software: PyCharm
===================================
"""
import base64
import os
import requests as req
from vgis_utils.vgis_datetime.datetimeTools import DateTimeHelper


class GeoserverOperatoer:

    # 初始化
    def __init__(self, geoserver_http_address, username, password):
        # "http://192.168.3.77:8085/geoserver"
        self.geoserver_http_address = geoserver_http_address
        self.serect = username + ":" + password
        self.basic_auth = "Basic " + str(base64.b64encode(self.serect.encode("utf-8")), "utf-8")

    # 清除服务缓存
    def clear_geoserver_cache(self):
        cmd = 'curl -v -u {} -X POST {}/rest/reset -H  "accept: application/json" -H  "content-type: application/json"'.format(
            self.serect, self.geoserver_http_address)
        print(cmd)
        os.system(cmd)

    # 发布geoserver tif 图层服务
    def publish_raster_layer_service(self, tif_path, workspacename, layer_name):
        print("发布栅格图层服务{}".format(layer_name))
        url = self.geoserver_http_address + "/rest/workspaces/" + workspacename + "/coveragestores/" + layer_name + "/vgis_file.geotiff"
        # Authorization信息在postman测试用例里看
        headers = {
            'Content-type': 'vgis_image/tiff',
            'Authorization': self.basic_auth
        }
        file = open(tif_path, 'rb')
        payload = file.read()
        response = req.request("PUT", url, headers=headers, data=payload)
        print(response.text)

    # 去除栅格图层的黑边
    def clear_black_raster_layer_service(self, workspacename, datastore_name, layer_name):

        print("去除黑边")
        url = "{}/rest/workspaces/{}/coveragestores/{}/coverages/{}".format(self.geoserver_http_address, workspacename,
                                                                            datastore_name, layer_name)

        payload = "<coverage>   \r\n    <parameters>\r\n    <entry>\r\n      <vgis_string>InputTransparentColor</vgis_string>\r\n      <vgis_string>#000000</vgis_string>\r\n    </entry>\r\n  </parameters>\r\n</coverage>\r\n"
        headers = {
            'Content-Type': 'application/xml',
            'Authorization': self.basic_auth
        }

        response = req.request("PUT", url, headers=headers, data=payload)

        print(response.text)

    # 判断是否有图层存在
    def is_has_layer(self, layer_name):
        ishas = False
        url = "{}/rest/layers/{}".format(self.geoserver_http_address, layer_name)
        payload = ""
        headers = {
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        if "No such layer" not in response.text:
            ishas = True

        return ishas

    # 发布geoserver shp 图层服务
    # TODO:有问题，针对部分shp不能自动识别空间范围和srs
    def publish_shp_layer_service(self, shp_path, layer_name, workspacename, layer_style_color):
        print("发布矢量图层服务{}".format(layer_name))
        url = self.geoserver_http_address + "/rest/workspaces/" + workspacename + "/datastores/" + layer_name + "/vgis_file.shp"
        file = open(shp_path, 'rb')
        payload = file.read()
        headers = {
            'Content-type': 'application/zip',
            'Authorization': self.basic_auth
        }
        response = req.request("PUT", url, headers=headers, data=payload)
        print(response.text)

    # 发布postgis矢量图层
    def publish_postgis_feature_service(self, layer_name, workspace_name, pg_datastore_name):
        cmd = 'curl -v -u {} -XPOST -H "Content-type: text/xml" -d  "<featureType><name>{}</name></featureType>" {}/rest/workspaces/{}/datastores/{}/featuretypes '.format(
            self.serect, layer_name, self.geoserver_http_address, workspace_name, pg_datastore_name)
        print(cmd)
        os.system(cmd)

    # 发布geoserver图层组服务
    def publish_layer_group_service(self, layer_group_name, workspacename, layer_name_list):
        print("发布图层组服务{}".format(layer_group_name))
        base_layer_name = layer_name_list[0]
        base_layer_info = self.get_layer_info(base_layer_name)
        layergroup_url = self.geoserver_http_address + "/rest/layergroups"
        layergroup_payload = "<layerGroup><name>" + layer_group_name + "</name><mode>SINGLE</mode><title>" + layer_group_name + "</title><workspace><name>" + workspacename + "</name></workspace>"
        layergroup_payload += "<layers>"
        for i in range(len(layer_name_list)):
            if layer_name_list[i] != None and layer_name_list[i].strip() != "":
                layergroup_payload += "<layer>" + layer_name_list[i] + "</layer>"
        layergroup_payload += "</layers>"
        layergroup_payload += "<styles>"
        for w in range(len(layer_name_list)):
            if layer_name_list[i] != None and layer_name_list[i].strip() != "":
                layergroup_payload += "<style>" + self.get_layer_style(layer_name_list[w]) + "</style>"
        layergroup_payload += "</styles>"
        layergroup_payload += "<bounds><minx>" + str(base_layer_info.get("minx")) + "</minx><maxx>" + str(
            base_layer_info.get("maxx")) + "</maxx><miny>" + str(base_layer_info.get("miny")) + "</miny><maxy>" + str(
            base_layer_info.get("maxy")) + "</maxy><crs>" + base_layer_info.get("srs") + "</crs></bounds></layerGroup>"
        print(layergroup_payload)
        layergroup_headers = {
            'Content-type': 'text/xml',
            'Authorization': self.basic_auth
        }
        curl_post_layergroup_response = req.request("POST", layergroup_url, headers=layergroup_headers,
                                                    data=layergroup_payload)
        print(curl_post_layergroup_response.text)

    # 得到图层信息，包括空间范围、坐标系等
    def get_layer_info(self, layer_name):
        url = self.geoserver_http_address + "/rest/layers/" + layer_name
        payload = {}
        headers = {
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        # print(response.text)
        response.encoding = 'utf-8'
        results = response.json()
        layer_prop_href = results.get("layer").get("resource").get("href")
        style_name = results.get("layer").get("defaultStyle").get("name")
        layer_response = req.request("GET", layer_prop_href, headers=headers)
        layer_response.encoding = 'utf-8'
        layer_results = layer_response.json()
        srs = layer_results.get("coverage").get("srs")
        minx = layer_results.get("coverage").get("nativeBoundingBox").get("minx")
        miny = layer_results.get("coverage").get("nativeBoundingBox").get("miny")
        maxx = layer_results.get("coverage").get("nativeBoundingBox").get("maxx")
        maxy = layer_results.get("coverage").get("nativeBoundingBox").get("maxy")
        obj = {}
        obj["style_name"] = style_name
        obj["srs"] = srs
        obj["minx"] = minx
        obj["miny"] = miny
        obj["maxx"] = maxx
        obj["maxy"] = maxy
        return obj

    # 获取图层的样式名称
    def get_layer_style(self, layer_name):
        url = self.geoserver_http_address + "/rest/layers/" + layer_name
        payload = {}
        headers = {
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        # print(response.text)
        response.encoding = 'utf-8'
        results = response.json()
        style_name = results.get("layer").get("defaultStyle").get("name")
        return style_name

    # 删除geoserver图层服务
    def delete_layer_service(self, layer_name):
        print("删除图层服务{}".format(layer_name))
        curl_delete_layer_url = self.geoserver_http_address + "/rest/layers/python:" + layer_name
        curl_delete_layer_payload = {}
        curl_delete_layerg_headers = {
            'Authorization': self.basic_auth
        }
        curl_delete_layer_response = req.request("DELETE", curl_delete_layer_url,
                                                 headers=curl_delete_layerg_headers,
                                                 data=curl_delete_layer_payload)
        print(curl_delete_layer_response.text)

    # 删除GeoServer图层
    def delete_layer_service2(self, layer_name):
        cmd = "curl -v -u admin:geoserver -XDELETE {}/rest/layers/{}?recurse=true".format(self.geoserver_http_address,
                                                                                          layer_name)
        os.system(cmd)

    # 删除geoserver图层组服务
    def delete_layer_group_service(self, layer_group_name, workspacename):
        print("删除图层组服务{}".format(layer_group_name))
        if layer_group_name != None and layer_group_name.strip() != "":
            curl_delete_layergroup_url = self.geoserver_http_address + "/rest/layergroups/" + workspacename + ":" + layer_group_name
            curl_delete_layergroup_payload = {}
            curl_delete_layergroup_headers = {
                'Authorization': self.basic_auth
            }
            curl_delete_layergroup_response = req.request("DELETE", curl_delete_layergroup_url,
                                                          headers=curl_delete_layergroup_headers,
                                                          data=curl_delete_layergroup_payload)
            print(curl_delete_layergroup_response.text)

    # 动态生成geoserver样式
    def make_layer_style(self, style_color, workspacename):
        style_name = "temp_style_" + DateTimeHelper.get_uuid()
        print("创建新样式{}".format(style_name))
        url = self.geoserver_http_address + "/rest/workspaces/" + workspacename + "/styles?name=" + style_name
        payload = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><sld:StyledLayerDescriptor xmlns=\"http://www.opengis.net/sld\" xmlns:sld=\"http://www.opengis.net/sld\" xmlns:gml=\"http://www.opengis.net/gml\" xmlns:ogc=\"http://www.opengis.net/ogc\" version=\"1.0.0\">\r\n  <sld:NamedLayer>\r\n    <sld:Name>polygon fill</sld:Name>\r\n    <sld:UserStyle>\r\n      <sld:Name>polygon fill</sld:Name>\r\n      <sld:Title>polygon fill</sld:Title>\r\n      <sld:Abstract>polygon fill</sld:Abstract>\r\n      <sld:FeatureTypeStyle>\r\n        <sld:Name>polygon fill</sld:Name>\r\n        <sld:Rule>\r\n          <sld:PolygonSymbolizer>\r\n            <sld:Fill>\r\n              <sld:CssParameter name=\"fill\">#66FF66</sld:CssParameter>\r\n              <sld:CssParameter name=\"fill-opacity\">0</sld:CssParameter>\r\n            </sld:Fill>\r\n            <sld:Stroke>\r\n              <sld:CssParameter name=\"stroke\">" + style_color + "</sld:CssParameter>\r\n            </sld:Stroke>\r\n          </sld:PolygonSymbolizer>\r\n        </sld:Rule>\r\n      </sld:FeatureTypeStyle>\r\n    </sld:UserStyle>\r\n  </sld:NamedLayer>\r\n</sld:StyledLayerDescriptor>\r\n"
        headers = {
            'accept': 'application/json',
            'content-type': 'application/vnd.ogc.sld+xml',
            'Authorization': self.basic_auth
        }
        response = req.request("POST", url, headers=headers, data=payload)
        print(response.text)
        return style_name

    # 删除geoerver样式
    def delete_layer_style(self, style_name, workspacename):
        print("删除样式{}".format(style_name))
        if style_name != None and style_name.strip() != "":
            url = self.geoserver_http_address + "/rest/styles/" + workspacename + ":" + style_name
            payload = {}
            headers = {
                'Authorization': self.basic_auth
            }
            response = req.request("DELETE", url, headers=headers, data=payload)
            print(response.text)

    # 给图层设置样式
    def set_style_to_layer(self, layer_name, style_name):
        print("为图层{}设置样式{}".format(layer_name, style_name))
        url = self.geoserver_http_address + "/rest/layers/" + layer_name
        payload = "<layer>\r\n    <defaultStyle>\r\n        <name>" + style_name + "</name>\r\n    </defaultStyle>\r\n    <enabled>true</enabled>\r\n</layer>"
        headers = {
            'Content-type': 'text/xml',
            'Authorization': self.basic_auth
        }
        response = req.request("PUT", url, headers=headers, data=payload)
        print(response.text)

    # 给图层设置样式
    def set_style_to_layer2(self, workspace_name, layer_name, style_name):
        cmd = 'curl -v -u {} -X PUT {}/rest/workspaces/{}/layers/{} -H "accept: application/json" -H "content-type: application/xml"  -d "<layer><defaultStyle><name>{}</name></defaultStyle></layer>"'.format(
            self.serect, workspace_name, layer_name, style_name)
        print(cmd)
        os.system(cmd)

    # 获取gwc切片方案信息
    def get_grid_set_info(self, grid_set_name):
        url = "{}/gwc/rest/gridsets/{}".format(self.geoserver_http_address, grid_set_name)
        payload = {}
        headers = {
            'accept': 'application/json',
            'content-type': 'application/json',
            'Authorization': self.basic_auth
        }
        response = req.request("GET", url, headers=headers, data=payload)
        print(response.text)
        return response.text
